/*! Tripetto Collector 1.7.1-26 - Copyright (C) 2019 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />
/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.metrics.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />

declare module "@marblecore/grid" {
    /** Implicit exports */
    export { Grid, TGrid } from "@marblecore/grid/lib/grid";
    export {
        TStructure as TGridStructure
    } from "@marblecore/grid/lib/structure";
    export { TResource as TGridResource } from "@marblecore/grid/lib/resource";
    export {
        Branches as GridBranches
    } from "@marblecore/grid/lib/branches/branches";
    export {
        Branch as GridBranch,
        TBranch as TGridBranch
    } from "@marblecore/grid/lib/branches/branch";
    export {
        Clusters as GridClusters
    } from "@marblecore/grid/lib/clusters/clusters";
    export {
        Cluster as GridCluster,
        TCluster as TGridCluster
    } from "@marblecore/grid/lib/clusters/cluster";
    export { Nodes as GridNodes } from "@marblecore/grid/lib/nodes/nodes";
    export {
        Node as GridNode,
        TNode as TGridNode
    } from "@marblecore/grid/lib/nodes/node";
    export { Layout as GridLayout } from "@marblecore/grid/lib/layout";
    export {
        Renderer as GridRenderer,
        ILayoutMapping as IGridLayout
    } from "@marblecore/grid/lib/renderer";
    export {
        TRectangles as TGridRectangles
    } from "@marblecore/grid/lib/rectangles";
    export { IResource as IGridResource } from "@marblecore/grid/lib/interface";
    export {
        IResizeEvent as IGridResizeEvent
    } from "@marblecore/grid/lib/events";
}

declare module "@marblecore/grid/lib/grid" {
    /** Dependencies */
    import { Stack } from "@marblecore/functional";
    import { THooks } from "@marblecore/hookup";
    import { Branch } from "@marblecore/grid/lib/branches/branch";
    import {
        IClusterConstructor,
        TCluster
    } from "@marblecore/grid/lib/clusters/cluster";
    import {
        IClustersConstructor,
        TClusters
    } from "@marblecore/grid/lib/clusters/clusters";
    import { IResource } from "@marblecore/grid/lib/interface";
    import { TNode } from "@marblecore/grid/lib/nodes/node";
    import { Renderer } from "@marblecore/grid/lib/renderer";
    export type TGrid = Grid<TClusters<TNode>, TNode, THooks>;
    export abstract class Grid<
        Children extends TClusters<T>,
        T extends TNode,
        Hooks extends THooks = void
    > extends Branch<Children, T, Hooks> {
        /**
         * Constructs a new `Grid` instance.
         * @param pClustersConstructor Reference to the `Clusters` constructor.
         * @param pClusterConstructor Reference to the `Cluster` constructor.
         * @param pRenderer Reference to a renderer.
         */
        constructor(
            pClustersConstructor: IClustersConstructor<T>,
            pClusterConstructor: IClusterConstructor<T, TCluster<T>>,
            pRenderer?: Renderer<TGrid>
        );
        /** Sets the overflow threshold for the grid. */
        threshold: number;
        /**
         * Deserializes to a grid.
         * @param pGrid Contains the serialized grid.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param sChildren Specified the data name for the child array.
         * @return Returns a reference to the instance.
         */
        deserialize(
            pGrid: IResource,
            pAfterwards: Stack,
            sChildren: string
        ): this;
    }
}

declare module "@marblecore/grid/lib/structure" {
    import { THooks } from "@marblecore/hookup";
    import { Item } from "@marblecore/matrix";
    import { Rectangle } from "@marblecore/metrics";
    import { TDispatcherFactory } from "@marblecore/grid/lib/dispatcher";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { Layout } from "@marblecore/grid/lib/layout";
    import { TRectangles } from "@marblecore/grid/lib/rectangles";
    import { Renderer } from "@marblecore/grid/lib/renderer";
    export type TStructure = Structure<
        Item<THooks>,
        TDispatcherFactory,
        TDispatcherFactory,
        THooks
    >;
    export abstract class Structure<
        Ancestor extends Item<THooks>,
        Dispatcher extends TDispatcherFactory,
        Children extends TDispatcherFactory,
        Hooks extends THooks
    > extends Item<Hooks> {
        /**
         * Constructs a new structure instance.
         * @param pDispatcher Reference to the dispatcher.
         * @param fnChildren Creates and returns the child structure.
         * @param pRenderer Reference to the renderer.
         * @param bRender Specifies if the structure needs to be rendered.
         */
        constructor(
            pDispatcher: Dispatcher | undefined,
            fnChildren:
                | ((
                      pParent: Structure<Ancestor, Dispatcher, Children, Hooks>
                  ) => Children)
                | undefined,
            pRenderer: Renderer<TGrid> | undefined
        );
        /** Retrieves the parent dispatcher. */
        readonly parent: Dispatcher | undefined;
        /** Retrieves the ancestor. */
        readonly ancestor: Ancestor | undefined;
        /** Retrieves the children of the structure. */
        readonly children: Children | undefined;
        /** Sets the zoom factor. */
        zoom: number;
        /** Retrieves the layout. */
        readonly layout:
            | Layout<TGrid, Structure<Ancestor, Dispatcher, Children, Hooks>>
            | undefined;
        /** Sets the renderer. */
        renderer: Renderer<TGrid> | undefined;
        /** Specifies if the structure is rendered. */
        readonly isRendered: boolean;
        /** Retrieves the rectangles of the structure. */
        readonly rectangles: TRectangles;
        /** Retrieves the rectangle of the structure. */
        readonly rectangle: Rectangle;
        /** Retrieves the left position of the structure. */
        readonly left: number;
        /** Retrieves the top position of the structure. */
        readonly top: number;
        /** Retrieves the right position of the structure. */
        readonly right: number;
        /** Retrieves the bottom position of the structure. */
        readonly bottom: number;
        /** Retrieves the width of the structure. */
        readonly width: number;
        /** Retrieves the height of the structure. */
        readonly height: number;
        /** Retrieves the horizontal mean of the structure. */
        readonly meanHorizontal: number;
        /** Retrieves the vertical mean of the structure. */
        readonly meanVertical: number;
        /** Invalidates the rendering. */
        protected renderInvalidate(): void;
        /**
         * Applies the position.
         * @param sType Specifies the change type.
         * @return Returns `true` if the structure position is changed.
         */
        protected applyPosition(sType?: "redraw" | "zoom"): boolean;
        /**
         * Applies a resize.
         * @param sType Specifies the resize type.
         * @return Returns `true` if the structure is resized.
         */
        applyResize(sType?: "redraw" | "zoom"): boolean;
        /**
         * Changes the dimensions of the structure.
         * @param nWidth Specifies the width of the structure.
         * @param nHeight Specifies the height of the structure.
         * @return Returns `true` if the structure is resized.
         */
        resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Iterate through each descendant of the structure.
         * @param fnStructure Specifies the structure function.
         * @param fnStructure.pStructure Contains a reference to the structure.
         * @return Returns `true` if the iteration is stopped (break).
         */
        abstract forEachDescendant(
            fnStructure: (pStructure: TStructure) => boolean | void
        ): boolean;
        /**
         * Iterate recursively through each descendant of the structure. For each
         * structure (including the structure from which the iteration originates)
         * a function will be invoked. This function should return `true` if the
         * structure needs to be iterated or `false` if the iteration should
         * continue with the next sibling structure.
         * @param fnStructure Specifies the structure function.
         * @param fnStructure.pStructure Contains a reference to the structure.
         * @return Returns the result of the structure function.
         */
        abstract forEachDescendantRecursive(
            fnStructure: (pStructure: TStructure) => boolean | void
        ): boolean;
        /**
         * Renders the structure.
         * @param bRecursive Specifies if the underlying structures needs to be
         * rendered.
         */
        render(bRecursive: boolean): void;
        /**
         * Invokes the layout renderer.
         * @param sType Specifies the rerender type. Can be one of the following values:
         * - `refresh`: Refresh the rendering;
         * - `update`: Update the rendering.
         */
        rerender(sType: "refresh" | "update"): void;
        /**
         * Invoked when the structure is rendered. Can be used to bind events.
         * @event
         */
        onRender(): void;
        /** Resets a structure. */
        reset(): void;
        /** Destroys a structure. */
        destroy(): void;
    }
}

declare module "@marblecore/grid/lib/resource" {
    /** Dependencies */
    import {
        TDispatcher,
        TDispatcherFactory
    } from "@marblecore/grid/lib/dispatcher";
    import { Stack } from "@marblecore/functional";
    import {
        IItemChangeEvent,
        IItemPopEvent,
        IItemPushEvent
    } from "@marblecore/matrix";
    import { IPoint } from "@marblecore/metrics";
    import { THooks } from "@marblecore/hookup";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { IResource } from "@marblecore/grid/lib/interface";
    import { Renderer } from "@marblecore/grid/lib/renderer";
    import { Structure, TStructure } from "@marblecore/grid/lib/structure";
    export type TResource = Resource<
        TStructure,
        TDispatcherFactory,
        TDispatcherFactory,
        THooks
    >;
    export abstract class Resource<
        Ancestor extends TStructure,
        Dispatcher extends TDispatcherFactory,
        Children extends TDispatcherFactory,
        Hooks extends THooks
    > extends Structure<Ancestor, Dispatcher, Children, Hooks> {
        /**
         * Constructs a new resource instance.
         * @param pDispatcher Reference to the dispatcher.
         * @param fnChildren Creates and returns the child resource.
         * @param pRenderer Reference to the renderer.
         */
        constructor(
            pDispatcher: Dispatcher | undefined,
            fnChildren?: (
                pParent: Resource<Ancestor, Dispatcher, Children, Hooks>
            ) => Children,
            pRenderer?: Renderer<TGrid>
        );
        /** Retrieves if this is the grid. */
        protected readonly isGrid: boolean;
        /** Retrieves the grid. */
        readonly grid: TGrid | undefined;
        /** Returns the z-index. */
        readonly z: number;
        /** Contains if the resource is being moved. */
        readonly isMoving: boolean;
        /**
         * Fired when a new resource is pushed to the dispatcher. This new resource
         * should be positioned, sized and drawn on the screen.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onPush(pEvent: IItemPushEvent<this>): void;
        /**
         * Fired when a resource is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onChange(pEvent: IItemChangeEvent<this>): void;
        /**
         * Fired when a resource is popped.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onPop(pEvent: IItemPopEvent<this>): void;
        /** Validates the resource. */
        protected onValidate(): void;
        /**
         * Iterate through each descendant of the resource.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns `true` if the iteration is stopped (break).
         */
        forEachDescendant(
            fnResource: (pResource: TResource) => boolean | void
        ): boolean;
        /**
         * Iterate recursively through each descendant of the resource. For each
         * resource (including the resource from which the iteration originates)
         * a function will be invoked. This function should return `true` if the
         * resource needs to be iterated or `false` if the iteration should
         * continue with the next sibling resource.
         * @param fnResource Specifies the resource function.
         * @param fnResource.pResource Contains a reference to the resource.
         * @return Returns the result of the resource function.
         */
        forEachDescendantRecursive(
            fnResource: (pResource: TResource) => boolean | void
        ): boolean;
        /**
         * Calculates the z-index of the resource.
         * @param nZ Specifies the minimum z-index.
         * @return Returns `true` if the z-index is changed.
         */
        zIndex(nZ?: number): boolean;
        /**
         * Calculates the position of the resource.
         * @param nLeft Specifies the left coordinate.
         * @param nTop Specifies the top coordinate.
         * @return Returns `true` if the position is changed.
         */
        position(nLeft?: number, nTop?: number): boolean;
        /**
         * Shifts the position of the resource.
         * @param nHorizontal Specifies the horizontal shift.
         * @param nVertical Specifies the vertical shift.
         */
        shift(nHorizontal: number, nVertical: number): void;
        /**
         * Calculates the dimensions of the resource.
         * @param nWidth Specifies the width of the resource.
         * @param nHeight Specifies the height of the resource.
         * @return Returns `true` if the resource is resized.
         */
        resize(nWidth?: number, nHeight?: number): boolean;
        /**
         * Moves the resource and returns the coordinates with the supplied offset
         * applied.
         * @param nLeft Specifies the left movement.
         * @param nTop Specifies the top movement.
         * @param pOffset Specifies the offset of the touch point in the element.
         * @return Returns the coordinates with the supplied offset.
         */
        move(nLeft?: number, nTop?: number, pOffset?: IPoint): IPoint;
        /** Requests a guides update and invokes the guides layout. */
        guides(): void;
        /** Fired when there is a child pushed or popped. */
        onPushOrPop(): void;
        /**
         * Fired when a resource move is initialized.
         * @param nDelay Specifies the delay before the move occurs.
         * @param pOffset Specifies the offset of the touch point.
         * @return Returns `true` if the move operation is initialized.
         * @event
         */
        onMoveInit(nDelay: number, pOffset: IPoint): boolean;
        /**
         * Fired when a resource move starts.
         * @param pOffset Specifies the offset of the touch point.
         * @return Returns `true` if the move operation is started.
         * @event
         */
        onMoveStart(pOffset: IPoint): boolean;
        /**
         * Fired when a resource move occurs.
         * @param nMoveX Specifies the horizontal movement.
         * @param nMoveY Specifies the vertical movement.
         * @param pOffset Specifies the offset of the touch point.
         * @param sDispatcher Specifies the name of the child dispatcher.
         * @event
         */
        onMove(
            nMoveX: number,
            nMoveY: number,
            pOffset: IPoint,
            sDispatcher?: string
        ): void;
        /**
         * Specifies the function which is invoked when a movement is validated.
         * @param pDispatcher Reference to the dispatcher.
         * @param nIndex Specifies the desired index for the resource.
         * @param bIsChild Specifies if the resource is already a child of the dispatcher.
         * @return Return `true` if the movement is validated.
         * @event
         */
        onMoveValidate(
            pDispatcher: TDispatcher,
            nIndex: number,
            bIsChild: boolean
        ): boolean;
        /**
         * Fired when a resource move ends.
         * @param bDragging Specifies if the `onMoveStart` event is fired.
         * @event
         */
        onMoveEnd(bDragging: boolean): void;
        /**
         * Generates a serialized representation of a resource.
         * @param sChildren Specified the data name for the child array.
         * @return Returns a reference to the serialized resource.
         */
        serialize(sChildren?: string): IResource;
        /**
         * Deserializes to a resource.
         * @param pResource Contains the serialized resource.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param sChildren Specified the data name for the child array.
         * @return Returns a reference to the instance.
         */
        deserialize(
            pResource: IResource,
            pAfterwards: Stack,
            sChildren?: string
        ): this;
        /** Resets the resource. */
        reset(): void;
        /** Destroys the resource and all of its children. */
        destroy(): void;
    }
}

declare module "@marblecore/grid/lib/branches/branches" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { TCluster } from "@marblecore/grid/lib/clusters/cluster";
    import { Dispatcher, TDispatcher } from "@marblecore/grid/lib/dispatcher";
    import { TNode } from "@marblecore/grid/lib/nodes/node";
    import {
        IBranchConstructor,
        TBranch
    } from "@marblecore/grid/lib/branches/branch";
    /** Branches template. */
    export type TBranches<T extends TNode> = Branches<TBranch<T>, T, THooks>;
    /** Branches constructor template. */
    export interface IBranchesConstructor<
        D extends TDispatcher,
        T extends TNode
    > {
        new (
            pBranchConstructor: IBranchConstructor<T, TBranch<T>>,
            pCluster: TCluster<T>
        ): D;
    }
    export abstract class Branches<
        Children extends TBranch<T>,
        T extends TNode,
        Hooks extends THooks = void
    > extends Dispatcher<TCluster<T>, Children, Hooks> {
        /**
         * Constructs a new `Branch` dispatcher.
         * @param pBranchConstructor Reference to the `Branch` constructor.
         * @param pCluster Specifies the parent `Cluster`.
         */
        constructor(
            pBranchConstructor: IBranchConstructor<T, Children>,
            pCluster: TCluster<T>
        );
    }
}

declare module "@marblecore/grid/lib/branches/branch" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import {
        IClusterConstructor,
        TCluster
    } from "@marblecore/grid/lib/clusters/cluster";
    import {
        IClustersConstructor,
        TClusters
    } from "@marblecore/grid/lib/clusters/clusters";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { TNode } from "@marblecore/grid/lib/nodes/node";
    import { Renderer } from "@marblecore/grid/lib/renderer";
    import { Resource } from "@marblecore/grid/lib/resource";
    import { TBranches } from "@marblecore/grid/lib/branches/branches";
    /** Cluster template. */
    export type TBranch<T extends TNode> = Branch<TClusters<T>, T, THooks>;
    /** Cluster constructor template. */
    export interface IBranchConstructor<T extends TNode, B extends TBranch<T>> {
        new (
            pBranches: TBranches<T>,
            pClustersConstructor: IClustersConstructor<T>,
            pClusterConstructor: IClusterConstructor<T, TCluster<T>>,
            pRenderer?: Renderer<TGrid>
        ): B;
    }
    export abstract class Branch<
        Children extends TClusters<T>,
        T extends TNode,
        Hooks extends THooks = void
    > extends Resource<TCluster<T>, TBranches<T>, Children, Hooks> {
        /**
         * Constructs a new `Branch` instance.
         * @param pBranches Reference to the parent `Branches`.
         * @param pClustersConstructor Reference to the `Clusters` constructor.
         * @param pClusterConstructor Reference to the `Cluster` constructor.
         * @param pRenderer Reference to a renderer.
         */
        constructor(
            pBranches: TBranches<T> | undefined,
            pClustersConstructor: IClustersConstructor<T>,
            pClusterConstructor: IClusterConstructor<T, TCluster<T>>,
            pRenderer?: Renderer<TGrid>
        );
    }
}

declare module "@marblecore/grid/lib/clusters/clusters" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { Branch, TBranch } from "@marblecore/grid/lib/branches/branch";
    import { Dispatcher } from "@marblecore/grid/lib/dispatcher";
    import { TNode } from "@marblecore/grid/lib/nodes/node";
    import {
        IClusterConstructor,
        TCluster
    } from "@marblecore/grid/lib/clusters/cluster";
    /** Clusters template. */
    export type TClusters<T extends TNode> = Clusters<TCluster<T>, T, THooks>;
    /** Clusters constructor template. */
    export interface IClustersConstructor<T extends TNode> {
        new (
            pClusterConstructor: IClusterConstructor<T, TCluster<T>>,
            pBranch: TBranch<T>,
            sOrientation?: "vertical" | "horizontal"
        ): TClusters<T>;
    }
    export abstract class Clusters<
        Children extends TCluster<T>,
        T extends TNode,
        Hooks extends THooks = void
    > extends Dispatcher<Branch<TClusters<T>, T, THooks>, Children, Hooks> {
        /**
         * Constructs a new `Cluster` dispatcher.
         * @param pClusterConstructor Reference to the `Cluster` constructor.
         * @param pBranch Specifies the parent `Branch`.
         * @param sOrientation Specifies the orientation of the grid.
         */
        constructor(
            pClusterConstructor: IClusterConstructor<T, Children>,
            pBranch: Branch<TClusters<T>, T, THooks>,
            sOrientation?: "vertical" | "horizontal"
        );
    }
}

declare module "@marblecore/grid/lib/clusters/cluster" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { Branch } from "@marblecore/grid/lib/branches/branch";
    import { INodeConstructor, TNode } from "@marblecore/grid/lib/nodes/node";
    import { INodesConstructor, Nodes } from "@marblecore/grid/lib/nodes/nodes";
    import { Resource } from "@marblecore/grid/lib/resource";
    import { TClusters } from "@marblecore/grid/lib/clusters/clusters";
    /** Cluster template. */
    export type TCluster<T extends TNode> = Cluster<
        Nodes<T, THooks>,
        T,
        THooks
    >;
    /** Cluster constructor template. */
    export interface IClusterConstructor<
        T extends TNode,
        C extends TCluster<T>
    > {
        new (
            pClusters: TClusters<T>,
            pNodesConstructor: INodesConstructor<T>,
            pNodeConstructor: INodeConstructor<T>
        ): C;
    }
    export abstract class Cluster<
        Children extends Nodes<T, THooks>,
        T extends TNode,
        Hooks extends THooks = void
    > extends Resource<
        Branch<TClusters<T>, T, THooks>,
        TClusters<T>,
        Children,
        Hooks
    > {
        /**
         * Constructs a new `Cluster` instance.
         * @param pClusters Reference to the parent `Clusters`.
         * @param pNodesConstructor Reference to the `Nodes` constructor.
         * @param pNodeConstructor Reference to the `Node` constructor.
         */
        constructor(
            pClusters: TClusters<T>,
            pNodesConstructor: INodesConstructor<T>,
            pNodeConstructor: INodeConstructor<T>
        );
    }
}

declare module "@marblecore/grid/lib/nodes/nodes" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { TAny } from "@marblecore/functional";
    import { TBranch } from "@marblecore/grid/lib/branches/branch";
    import { TCluster } from "@marblecore/grid/lib/clusters/cluster";
    import { Dispatcher } from "@marblecore/grid/lib/dispatcher";
    import { INodeConstructor, TNode } from "@marblecore/grid/lib/nodes/node";
    /** Nodes template. */
    export type TNodes<T extends TNode> = Nodes<T, TAny>;
    /** Nodes constructor template. */
    export interface INodesConstructor<T extends TNode> {
        new (
            pNodeConstructor: INodeConstructor<T>,
            pParent: TBranch<T> | TCluster<T>,
            nDimensions?: number,
            sOrientation?: "vertical" | "horizontal"
        ): Nodes<T, THooks>;
    }
    export abstract class Nodes<
        T extends TNode,
        Hooks extends THooks = void
    > extends Dispatcher<TBranch<TNode> | TCluster<TNode>, T, Hooks> {
        /**
         * Constructs a new `Node` dispatcher.
         * @param pNodeConstructor Reference to the `Node` constructor.
         * @param pParent Specifies the parent `Branch` or `Cluster`.
         * @param nDimensions Specifies the number of columns for vertical grids
         * or the number of rows for horizontal grids.
         * @param sOrientation Specifies the orientation of the grid.
         */
        constructor(
            pNodeConstructor: INodeConstructor<T>,
            pParent: TBranch<TNode> | TCluster<TNode>,
            nDimensions?: number,
            sOrientation?: "vertical" | "horizontal"
        );
    }
}

declare module "@marblecore/grid/lib/nodes/node" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { TBranch } from "@marblecore/grid/lib/branches/branch";
    import { TCluster } from "@marblecore/grid/lib/clusters/cluster";
    import { TDispatcher } from "@marblecore/grid/lib/dispatcher";
    import { Resource } from "@marblecore/grid/lib/resource";
    import { Nodes } from "@marblecore/grid/lib/nodes/nodes";
    /** Node template. */
    export type TNode = Node<THooks>;
    /** Node constructor template. */
    export interface INodeConstructor<T extends TNode> {
        new (pNodes: Nodes<T, THooks>): T;
    }
    export abstract class Node<Hooks extends THooks = void> extends Resource<
        TCluster<Node<Hooks>> | TBranch<Node<Hooks>>,
        Nodes<Node<Hooks>, THooks>,
        TDispatcher,
        Hooks
    > {
        /**
         * Constructs a new `Node` instance.
         * @param pNodes Reference to the parent `Nodes`.
         */
        constructor(pNodes: Nodes<Node<Hooks>, THooks>);
    }
}

declare module "@marblecore/grid/lib/layout" {
    /** Dependencies */
    import { Coordinate, IPoint } from "@marblecore/metrics";
    import { TAny } from "@marblecore/functional";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { TRectangles } from "@marblecore/grid/lib/rectangles";
    import { Renderer } from "@marblecore/grid/lib/renderer";
    import { TResource } from "@marblecore/grid/lib/resource";
    import { TStructure } from "@marblecore/grid/lib/structure";
    export interface ILayout<Grid extends TGrid, Parent extends TStructure> {
        /**
         * Specifies the constructor function.
         * @param pParent Specifies a reference to the parent.
         * @param pArguments Specifies additional constructor arguments.
         */
        new (
            pRenderer: Renderer<Grid>,
            pParent: Parent,
            ...pArguments: TAny[]
        ): Layout<Grid, Parent>;
    }
    export abstract class Layout<
        Grid extends TGrid,
        Parent extends TStructure
    > {
        /**
         * Constructs a new layout.
         * @param pRenderer Reference to the renderer.
         * @param pParent Reference to the layout parent.
         */
        constructor(pRenderer: Renderer<Grid>, pParent: Parent);
        /** Returns the layout configuration. */
        abstract readonly configuration: TRectangles;
        /** Retrieves a reference to the renderer. */
        readonly renderer: Renderer<Grid>;
        /** Retrieves a reference to the layout parent. */
        readonly parent: Parent;
        /** Retrieves the horizontal offset for the layout. */
        readonly x: number;
        /** Retrieves the vertical offset for the layout. */
        readonly y: number;
        /**
         * Invooked when the layout is about to resize.
         * @param pRectangles Contains the layout rectangles.
         */
        beforeResize(pRectangles: TRectangles): void;
        /**
         * Fired when the z-index is changed.
         * @param nZ Specifies the z-index.
         * @event
         */
        zIndex(nZ: number): void;
        /**
         * Fired when the layout should be elevated.
         * @param bElevate Specifies if the layout is elevated.
         * @event
         */
        onElevate(bElevate: boolean): void;
        /**
         * Fired when an item becomes part of a dispatcher.
         * @event
         */
        onPush(): void;
        /**
         * Fired when an item becomes the first item.
         * @event
         */
        onFirst(): void;
        /**
         * Fired when an item becomes the last item.
         * @event
         */
        onLast(): void;
        /**
         * Specifies the expansion coordinates for the move detection. These
         * coordinates are applied to the rectangle detection. Use it to
         * expand or contract the rectangle.
         * @param nX Specifies the horizontal mouse coordinate.
         * @param nY Specifies the vertical mouse coordinate.
         * @param pTarget Specifies the target resource.
         * @return Returns the expansion coordinates.
         * @event
         */
        onMoveExpansion(nX: number, nY: number, pTarget: TResource): Coordinate;
        /**
         * Fired when the system wants to know if the supplied mouse coordinates
         * are within the resource.
         * @param nX Specifies the horizontal mouse coordinate.
         * @param nY Specifies the vertical mouse coordinate.
         * @param pTarget Specifies the target resource.
         * @return Returns `true` if the mouse is within the resource.
         * @event
         */
        onIsMoveIn(nX: number, nY: number, pTarget: TResource): boolean;
        /**
         * Specifies the function which is invoked when the move is initialized.
         * @param nDelay Specifies the delay in milliseconds before the movement starts.
         * @param pOffset Contains the touch/click offset relative to the resource.
         * @return Return `true` if the move should be continued.
         * @event
         */
        onMoveInit(nDelay: number, pOffset: IPoint): boolean;
        /**
         * Specifies the function which is invoked when the move starts.
         * @param pOffset Contains the touch/click offset relative to the resource.
         * @return Return `true` if the move should be continued.
         * @event
         */
        onMoveStart(pOffset: IPoint): boolean;
        /**
         * Specifies the function which is invoked when a movement occurs.
         * @param nMouseX Specifies the horizontal mouse coordinate.
         * @param nMouseY Specifies the vertical mouse coordinate.
         * @param bPopped Specifies if the resource is popped. In that case the
         * resource is not inside a parent which can host the resource.
         * @param pOffset Contains the touch/click offset relative to the resource.
         * @return Return `true` if the movement is valid. If you return `false`
         * the item will be popped.
         * @event
         */
        onMove(
            nMouseX: number,
            nMouseY: number,
            bPopped: boolean,
            pOffset: IPoint
        ): boolean;
        /**
         * Specifies the function which is invoked when the move ends.
         * @param bDragging Specifies if the `MoveStart` event is fired and the
         * resource was in dragging mode.
         * @event
         */
        onMoveEnd(bDragging: boolean): void;
        /**
         * Invoked when the guides of a rendering are changed.
         * @param bVisible Indicates if the guides should be visible.
         * @param nLeft Specifies the left coordinate of the guides.
         * @param nTop Specifies the top coordinate of the guides.
         * @param nWidth Specifies the width of the guides.
         * @param nHeight Specifies the height of the guides.
         * @param nZoom Specifies the zoom factor of the guides.
         * @event
         */
        onGuidesChange(
            bVisible: boolean,
            nLeft: number,
            nTop: number,
            nWidth: number,
            nHeight: number,
            nZoom: number
        ): void;
        /**
         * Renders the layout.
         * @param sType Specifies the render type. Can be one of the following values:
         * - `render`: Do a complete render;
         * - `refresh`: Refresh the rendering;
         * - `update`: Update the rendering.
         */
        abstract render(sType: "render" | "refresh" | "update"): void;
        /**
         * Sets the position of the layout element.
         * @param nLeft Specifies the horizontal position.
         * @param nTop Specifies the vertical position.
         * @param nZoom Contains the zoom factor.
         * @param sReason Specifies the reason for the position change.
         */
        abstract position(
            nLeft: number,
            nTop: number,
            nZoom: number,
            sReason: "redraw" | "zoom"
        ): void;
        /**
         * Sets the dimensions of the layout element.
         * @param nWidth Specifies the width.
         * @param nHeight Specifies the height.
         * @param nZoom Contains the zoom factor.
         * @param sReason Specifies the reason for the resize.
         */
        abstract resize(
            nWidth: number,
            nHeight: number,
            nZoom: number,
            sReason: "redraw" | "width" | "height" | "zoom"
        ): void;
        /** Destroys the layout. */
        abstract destroy(): void;
    }
}

declare module "@marblecore/grid/lib/renderer" {
    /** Dependencies */
    import { TAny } from "@marblecore/functional";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { ILayout, Layout } from "@marblecore/grid/lib/layout";
    import { TStructure } from "@marblecore/grid/lib/structure";
    export interface IType<T extends TStructure> {
        /**
         * Specifies the constructor function.
         * @param pArguments Specifies additional constructor arguments.
         */
        new (...pArguments: TAny[]): T;
    }
    export interface ILayoutMapping<
        Grid extends TGrid,
        Type extends TStructure
    > {
        /** Specifies the type constructor. */
        type: IType<Type>;
        /** Specifies the layout. */
        layout: ILayout<Grid, Type>;
    }
    export abstract class Renderer<
        Grid extends TGrid,
        Types extends TStructure = TStructure
    > {
        /**
         * Constructs a new renderer.
         * @param pLayouts Specifies an array of layouts which are rendered.
         */
        constructor(...pLayouts: ILayoutMapping<Grid, Types>[]);
        /** Sets the rendering grid. */
        grid: Grid | undefined;
        /**
         * Provides the layout constructor for the specified parent type.
         * @param pType Reference to the parent type.
         * @return Returns the layout provider.
         */
        protected provider<Type extends TStructure>(
            pType: Type
        ): ILayoutMapping<Grid, Type>;
        /**
         * Retrieves a new layout instance for the specified type.
         * @param pType Reference to the parent type.
         * @return Returns the layout instance.
         */
        abstract layout(pType: Types): Layout<Grid, Types>;
        /**
         * Fired when the renderer is initialized.
         * @event
         */
        onInit(): void;
        /**
         * Fired when the dimensions of the rendering are changed.
         * @param nWidth Specifies the width.
         * @param nHeight Specifies the height.
         * @param nZoom Contains the zoom factor.
         * @param sReason Specifies the reason for the resize.
         * @event
         */
        onResize(
            nWidth: number,
            nHeight: number,
            nZoom: number,
            sReason: "redraw" | "width" | "height" | "zoom"
        ): void;
        /**
         * Fired when a move starts.
         * @event
         */
        onMoveStart(): void;
        /**
         * Fired when something is moving.
         * @event
         */
        onMove(): void;
        /**
         * Fired when the moving ends.
         * @event
         */
        onMoveEnd(): void;
    }
}

declare module "@marblecore/grid/lib/rectangles" {
    import { Rectangles } from "@marblecore/metrics";
    /** Specifies the rectangles type. */
    export type TRectangles = Rectangles<"self" | "children">;
}

declare module "@marblecore/grid/lib/interface" {
    /** Dependencies */
    import { ISerialize } from "@marblecore/serializer";
    export interface IResource extends ISerialize {
        /** Contains the identifier for the serialized resource. */
        readonly id: string;
    }
}

declare module "@marblecore/grid/lib/events" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { TDispatcher } from "@marblecore/grid/lib/dispatcher";
    export interface IResizeEvent extends IHookPayload<"OnResize"> {
        /** Reference to the dispatcher. */
        readonly dispatcher: TDispatcher;
    }
}

declare module "@marblecore/grid/lib/dispatcher" {
    /** Dependencies */
    import { Stack, TAny } from "@marblecore/functional";
    import { THooks } from "@marblecore/hookup";
    import {
        ICountChangeEvent,
        IDimensionChangeEvent,
        Matrix,
        Orientations,
        Types
    } from "@marblecore/matrix";
    import { TGrid } from "@marblecore/grid/lib/grid";
    import { IResource } from "@marblecore/grid/lib/interface";
    import { TResource } from "@marblecore/grid/lib/resource";
    export type TDispatcher = Dispatcher<TResource, TResource, THooks>;
    export type TDispatcherFactory = Dispatcher<TAny, TAny, THooks>;
    export type TDispatcherEvents<Hooks extends THooks> = Hooks | "OnResize";
    /** Dispatcher constructor template. */
    export interface IDispatcherConstructor<
        Parent extends TResource,
        T extends TResource,
        Hooks extends THooks
    > {
        new (
            pDispatcher: Dispatcher<Parent, T, Hooks>,
            ...pArguments: TAny[]
        ): T;
    }
    export abstract class Dispatcher<
        Parent extends TResource,
        T extends TResource,
        Hooks extends THooks
    > extends Matrix<T, TDispatcherEvents<Hooks>> {
        /**
         * Constructs a new dispatcher.
         * @param pType Reference to the dispatcher type.
         * @param pParent Reference to the parent (this is the ancestor of an item).
         * @param nType Specifies the direction.
         * @param nOrientation Specifies the orientation.
         * @param nSize Specifies the size.
         */
        constructor(
            pType: IDispatcherConstructor<Parent, T, Hooks>,
            pParent: Parent,
            nType: Types,
            nOrientation: Orientations,
            nSize: number
        );
        /** Retrieves the grid. */
        readonly grid: TGrid | undefined;
        /** Retrieves the dispatcher type. */
        readonly dispatcher: IDispatcherConstructor<Parent, T, Hooks>;
        /** Returns a reference to the parent. */
        readonly parent: Parent;
        /**
         * Appends an item to the end.
         * @param pItem Reference to the item.
         * @return Returns a reference to the item.
         */
        protected appendItem(pItem: T): T;
        /**
         * Inserts an item at the begin.
         * @param pItem Reference to the item.
         * @return Returns a reference to the item.
         */
        protected insertItem(pItem: T): T;
        /**
         * Inserts an item at the specified index.
         * @param pItem Reference to the item.
         * @param nIndex Specifies the index position.
         * @return Returns a reference to the item.
         */
        protected insertItemAtIndex(pItem: T, nIndex: number): T;
        /**
         * Fired when the dimensions of the dispatcher are changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onDimensionChange(pEvent: IDimensionChangeEvent<this>): void;
        protected onCountChange(pEvent: ICountChangeEvent<this>): void;
        /**
         * Invoked when the dispatcher dimensions are changed.
         * @event
         */
        onDimensions(): void;
        /**
         * Deserializes the supplied resources to the dispatcher.
         * @param pAfterwards Stack with calls which are invoked after the
         * structural deserialization is done.
         * @param pResources Reference to the resources.
         */
        deserialize(pAfterwards: Stack, ...pResources: IResource[]): void;
        /** Resets a dispatcher. */
        reset(): void;
        /** Destroys a dispatcher. */
        destroy(): void;
    }
}
